# -*- coding: utf-8 -*-
"""
A list of Ecuador departaments as `choices` in a
formfield.

This exists in this standalone file so that it's only
imported into memory when explicitly needed.
"""

PROVINCE_CHOICES = (
    ('A', u'Azuay'),
    ('B', u'Bolívar'),
    ('F', u'Cañar'),
    ('C', u'Carchi'),
    ('H', u'Chimborazo'),
    ('X', u'Cotopaxi'),
    ('O', u'El Oro'),
    ('E', u'Esmeraldas'),
    ('W', u'Galápagos'),
    ('G', u'Guayas'),
    ('I', u'Imbabura'),
    ('L', u'Loja'),
    ('R', u'Los Ríos'),
    ('M', u'Manabí'),
    ('S', u'Morona Santiago'),
    ('N', u'Napo'),
    ('D', u'Orellana'),
    ('Y', u'Pastaza'),
    ('P', u'Pichincha'),
    ('SE', u'Santa Elena'),
    ('SD', u'Santo Domingo de los Tsáchilas'),
    ('U', u'Sucumbíos'),
    ('T', u'Tungurahua'),
    ('Z', u'Zamora Chinchipe'),
)
