import warnings
from django.contrib.databrowse.sites import DatabrowsePlugin, ModelDatabrowse, DatabrowseSite, site


warnings.warn("The Databrowse contrib app is deprecated",
    PendingDeprecationWarning)
