# -*- coding: utf-8 -*-
"""
Polish administrative units as in http://pl.wikipedia.org/wiki/Podzia%C5%82_administracyjny_Polski
"""


ADMINISTRATIVE_UNIT_CHOICES = (
    ('wroclaw', u'Wrocław'),
    ('jeleniagora', u'Jelenia Góra'),
    ('legnica', u'Legnica'),
    ('boleslawiecki', u'bolesławiecki'),
    ('dzierzoniowski', u'dzierżoniowski'),
    ('glogowski', u'głogowski'),
    ('gorowski', u'górowski'),
    ('jaworski', u'jaworski'),
    ('jeleniogorski', u'jeleniogórski'),
    ('kamiennogorski', u'kamiennogórski'),
    ('klodzki', u'kłodzki'),
    ('legnicki', u'legnicki'),
    ('lubanski', u'lubański'),
    ('lubinski', u'lubiński'),
    ('lwowecki', u'lwówecki'),
    ('milicki', u'milicki'),
    ('olesnicki', u'oleśnicki'),
    ('olawski', u'oławski'),
    ('polkowicki', u'polkowicki'),
    ('strzelinski', u'strzeliński'),
    ('sredzki', u'średzki'),
    ('swidnicki', u'świdnicki'),
    ('trzebnicki', u'trzebnicki'),
    ('walbrzyski', u'wałbrzyski'),
    ('wolowski', u'wołowski'),
    ('wroclawski', u'wrocławski'),
    ('zabkowicki', u'ząbkowicki'),
    ('zgorzelecki', u'zgorzelecki'),
    ('zlotoryjski', u'złotoryjski'),
    ('bydgoszcz', u'Bydgoszcz'),
    ('torun', u'Toruń'),
    ('wloclawek', u'Włocławek'),
    ('grudziadz', u'Grudziądz'),
    ('aleksandrowski', u'aleksandrowski'),
    ('brodnicki', u'brodnicki'),
    ('bydgoski', u'bydgoski'),
    ('chelminski', u'chełmiński'),
    ('golubsko-dobrzynski', u'golubsko-dobrzyński'),
    ('grudziadzki', u'grudziądzki'),
    ('inowroclawski', u'inowrocławski'),
    ('lipnowski', u'lipnowski'),
    ('mogilenski', u'mogileński'),
    ('nakielski', u'nakielski'),
    ('radziejowski', u'radziejowski'),
    ('rypinski', u'rypiński'),
    ('sepolenski', u'sępoleński'),
    ('swiecki', u'świecki'),
    ('torunski', u'toruński'),
    ('tucholski', u'tucholski'),
    ('wabrzeski', u'wąbrzeski'),
    ('wloclawski', u'wrocławski'),
    ('zninski', u'źniński'),
    ('lublin', u'Lublin'),
    ('biala-podlaska', u'Biała Podlaska'),
    ('chelm', u'Chełm'),
    ('zamosc', u'Zamość'),
    ('bialski', u'bialski'),
    ('bilgorajski', u'biłgorajski'),
    ('chelmski', u'chełmski'),
    ('hrubieszowski', u'hrubieszowski'),
    ('janowski', u'janowski'),
    ('krasnostawski', u'krasnostawski'),
    ('krasnicki', u'kraśnicki'),
    ('lubartowski', u'lubartowski'),
    ('lubelski', u'lubelski'),
    ('leczynski', u'łęczyński'),
    ('lukowski', u'łukowski'),
    ('opolski', u'opolski'),
    ('parczewski', u'parczewski'),
    ('pulawski', u'puławski'),
    ('radzynski', u'radzyński'),
    ('rycki', u'rycki'),
    ('swidnicki', u'świdnicki'),
    ('tomaszowski', u'tomaszowski'),
    ('wlodawski', u'włodawski'),
    ('zamojski', u'zamojski'),
    ('gorzow-wielkopolski', u'Gorzów Wielkopolski'),
    ('zielona-gora', u'Zielona Góra'),
    ('gorzowski', u'gorzowski'),
    ('krosnienski', u'krośnieński'),
    ('miedzyrzecki', u'międzyrzecki'),
    ('nowosolski', u'nowosolski'),
    ('slubicki', u'słubicki'),
    ('strzelecko-drezdenecki', u'strzelecko-drezdenecki'),
    ('sulecinski', u'suleńciński'),
    ('swiebodzinski', u'świebodziński'),
    ('wschowski', u'wschowski'),
    ('zielonogorski', u'zielonogórski'),
    ('zaganski', u'żagański'),
    ('zarski', u'żarski'),
    ('lodz', u'Łódź'),
    ('piotrkow-trybunalski', u'Piotrków Trybunalski'),
    ('skierniewice', u'Skierniewice'),
    ('belchatowski', u'bełchatowski'),
    ('brzezinski', u'brzeziński'),
    ('kutnowski', u'kutnowski'),
    ('laski', u'łaski'),
    ('leczycki', u'łęczycki'),
    ('lowicki', u'łowicki'),
    ('lodzki wschodni', u'łódzki wschodni'),
    ('opoczynski', u'opoczyński'),
    ('pabianicki', u'pabianicki'),
    ('pajeczanski', u'pajęczański'),
    ('piotrkowski', u'piotrkowski'),
    ('poddebicki', u'poddębicki'),
    ('radomszczanski', u'radomszczański'),
    ('rawski', u'rawski'),
    ('sieradzki', u'sieradzki'),
    ('skierniewicki', u'skierniewicki'),
    ('tomaszowski', u'tomaszowski'),
    ('wielunski', u'wieluński'),
    ('wieruszowski', u'wieruszowski'),
    ('zdunskowolski', u'zduńskowolski'),
    ('zgierski', u'zgierski'),
    ('krakow', u'Kraków'),
    ('tarnow', u'Tarnów'),
    ('nowy-sacz', u'Nowy Sącz'),
    ('bochenski', u'bocheński'),
    ('brzeski', u'brzeski'),
    ('chrzanowski', u'chrzanowski'),
    ('dabrowski', u'dąbrowski'),
    ('gorlicki', u'gorlicki'),
    ('krakowski', u'krakowski'),
    ('limanowski', u'limanowski'),
    ('miechowski', u'miechowski'),
    ('myslenicki', u'myślenicki'),
    ('nowosadecki', u'nowosądecki'),
    ('nowotarski', u'nowotarski'),
    ('olkuski', u'olkuski'),
    ('oswiecimski', u'oświęcimski'),
    ('proszowicki', u'proszowicki'),
    ('suski', u'suski'),
    ('tarnowski', u'tarnowski'),
    ('tatrzanski', u'tatrzański'),
    ('wadowicki', u'wadowicki'),
    ('wielicki', u'wielicki'),
    ('warszawa', u'Warszawa'),
    ('ostroleka', u'Ostrołęka'),
    ('plock', u'Płock'),
    ('radom', u'Radom'),
    ('siedlce', u'Siedlce'),
    ('bialobrzeski', u'białobrzeski'),
    ('ciechanowski', u'ciechanowski'),
    ('garwolinski', u'garwoliński'),
    ('gostyninski', u'gostyniński'),
    ('grodziski', u'grodziski'),
    ('grojecki', u'grójecki'),
    ('kozienicki', u'kozenicki'),
    ('legionowski', u'legionowski'),
    ('lipski', u'lipski'),
    ('losicki', u'łosicki'),
    ('makowski', u'makowski'),
    ('minski', u'miński'),
    ('mlawski', u'mławski'),
    ('nowodworski', u'nowodworski'),
    ('ostrolecki', u'ostrołęcki'),
    ('ostrowski', u'ostrowski'),
    ('otwocki', u'otwocki'),
    ('piaseczynski', u'piaseczyński'),
    ('plocki', u'płocki'),
    ('plonski', u'płoński'),
    ('pruszkowski', u'pruszkowski'),
    ('przasnyski', u'przasnyski'),
    ('przysuski', u'przysuski'),
    ('pultuski', u'pułtuski'),
    ('radomski', u'radomski'),
    ('siedlecki', u'siedlecki'),
    ('sierpecki', u'sierpecki'),
    ('sochaczewski', u'sochaczewski'),
    ('sokolowski', u'sokołowski'),
    ('szydlowiecki', u'szydłowiecki'),
    ('warszawski-zachodni', u'warszawski zachodni'),
    ('wegrowski', u'węgrowski'),
    ('wolominski', u'wołomiński'),
    ('wyszkowski', u'wyszkowski'),
    ('zwolenski', u'zwoleński'),
    ('zurominski', u'żuromiński'),
    ('zyrardowski', u'żyrardowski'),
    ('opole', u'Opole'),
    ('brzeski', u'brzeski'),
    ('glubczycki', u'głubczyski'),
    ('kedzierzynsko-kozielski', u'kędzierzyński-kozielski'),
    ('kluczborski', u'kluczborski'),
    ('krapkowicki', u'krapkowicki'),
    ('namyslowski', u'namysłowski'),
    ('nyski', u'nyski'),
    ('oleski', u'oleski'),
    ('opolski', u'opolski'),
    ('prudnicki', u'prudnicki'),
    ('strzelecki', u'strzelecki'),
    ('rzeszow', u'Rzeszów'),
    ('krosno', u'Krosno'),
    ('przemysl', u'Przemyśl'),
    ('tarnobrzeg', u'Tarnobrzeg'),
    ('bieszczadzki', u'bieszczadzki'),
    ('brzozowski', u'brzozowski'),
    ('debicki', u'dębicki'),
    ('jaroslawski', u'jarosławski'),
    ('jasielski', u'jasielski'),
    ('kolbuszowski', u'kolbuszowski'),
    ('krosnienski', u'krośnieński'),
    ('leski', u'leski'),
    ('lezajski', u'leżajski'),
    ('lubaczowski', u'lubaczowski'),
    ('lancucki', u'łańcucki'),
    ('mielecki', u'mielecki'),
    ('nizanski', u'niżański'),
    ('przemyski', u'przemyski'),
    ('przeworski', u'przeworski'),
    ('ropczycko-sedziszowski', u'ropczycko-sędziszowski'),
    ('rzeszowski', u'rzeszowski'),
    ('sanocki', u'sanocki'),
    ('stalowowolski', u'stalowowolski'),
    ('strzyzowski', u'strzyżowski'),
    ('tarnobrzeski', u'tarnobrzeski'),
    ('bialystok', u'Białystok'),
    ('lomza', u'Łomża'),
    ('suwalki', u'Suwałki'),
    ('augustowski', u'augustowski'),
    ('bialostocki', u'białostocki'),
    ('bielski', u'bielski'),
    ('grajewski', u'grajewski'),
    ('hajnowski', u'hajnowski'),
    ('kolnenski', u'kolneński'),
    ('łomzynski', u'łomżyński'),
    ('moniecki', u'moniecki'),
    ('sejnenski', u'sejneński'),
    ('siemiatycki', u'siematycki'),
    ('sokolski', u'sokólski'),
    ('suwalski', u'suwalski'),
    ('wysokomazowiecki', u'wysokomazowiecki'),
    ('zambrowski', u'zambrowski'),
    ('gdansk', u'Gdańsk'),
    ('gdynia', u'Gdynia'),
    ('slupsk', u'Słupsk'),
    ('sopot', u'Sopot'),
    ('bytowski', u'bytowski'),
    ('chojnicki', u'chojnicki'),
    ('czluchowski', u'człuchowski'),
    ('kartuski', u'kartuski'),
    ('koscierski', u'kościerski'),
    ('kwidzynski', u'kwidzyński'),
    ('leborski', u'lęborski'),
    ('malborski', u'malborski'),
    ('nowodworski', u'nowodworski'),
    ('gdanski', u'gdański'),
    ('pucki', u'pucki'),
    ('slupski', u'słupski'),
    ('starogardzki', u'starogardzki'),
    ('sztumski', u'sztumski'),
    ('tczewski', u'tczewski'),
    ('wejherowski', u'wejcherowski'),
    ('katowice', u'Katowice'),
    ('bielsko-biala', u'Bielsko-Biała'),
    ('bytom', u'Bytom'),
    ('chorzow', u'Chorzów'),
    ('czestochowa', u'Częstochowa'),
    ('dabrowa-gornicza', u'Dąbrowa Górnicza'),
    ('gliwice', u'Gliwice'),
    ('jastrzebie-zdroj', u'Jastrzębie Zdrój'),
    ('jaworzno', u'Jaworzno'),
    ('myslowice', u'Mysłowice'),
    ('piekary-slaskie', u'Piekary Śląskie'),
    ('ruda-slaska', u'Ruda Śląska'),
    ('rybnik', u'Rybnik'),
    ('siemianowice-slaskie', u'Siemianowice Śląskie'),
    ('sosnowiec', u'Sosnowiec'),
    ('swietochlowice', u'Świętochłowice'),
    ('tychy', u'Tychy'),
    ('zabrze', u'Zabrze'),
    ('zory', u'Żory'),
    ('bedzinski', u'będziński'),
    ('bielski', u'bielski'),
    ('bierunsko-ledzinski', u'bieruńsko-lędziński'),
    ('cieszynski', u'cieszyński'),
    ('czestochowski', u'częstochowski'),
    ('gliwicki', u'gliwicki'),
    ('klobucki', u'kłobucki'),
    ('lubliniecki', u'lubliniecki'),
    ('mikolowski', u'mikołowski'),
    ('myszkowski', u'myszkowski'),
    ('pszczynski', u'pszczyński'),
    ('raciborski', u'raciborski'),
    ('rybnicki', u'rybnicki'),
    ('tarnogorski', u'tarnogórski'),
    ('wodzislawski', u'wodzisławski'),
    ('zawiercianski', u'zawierciański'),
    ('zywiecki', u'żywiecki'),
    ('kielce', u'Kielce'),
    ('buski', u'buski'),
    ('jedrzejowski', u'jędrzejowski'),
    ('kazimierski', u'kazimierski'),
    ('kielecki', u'kielecki'),
    ('konecki', u'konecki'),
    ('opatowski', u'opatowski'),
    ('ostrowiecki', u'ostrowiecki'),
    ('pinczowski', u'pińczowski'),
    ('sandomierski', u'sandomierski'),
    ('skarzyski', u'skarżyski'),
    ('starachowicki', u'starachowicki'),
    ('staszowski', u'staszowski'),
    ('wloszczowski', u'włoszczowski'),
    ('olsztyn', u'Olsztyn'),
    ('elblag', u'Elbląg'),
    ('bartoszycki', u'bartoszycki'),
    ('braniewski', u'braniewski'),
    ('dzialdowski', u'działdowski'),
    ('elblaski', u'elbląski'),
    ('elcki', u'ełcki'),
    ('gizycki', u'giżycki'),
    ('goldapski', u'gołdapski'),
    ('ilawski', u'iławski'),
    ('ketrzynski', u'kętrzyński'),
    ('lidzbarski', u'lidzbarski'),
    ('mragowski', u'mrągowski'),
    ('nidzicki', u'nidzicki'),
    ('nowomiejski', u'nowomiejski'),
    ('olecki', u'olecki'),
    ('olsztynski', u'olsztyński'),
    ('ostrodzki', u'ostródzki'),
    ('piski', u'piski'),
    ('szczycienski', u'szczycieński'),
    ('wegorzewski', u'węgorzewski'),
    ('poznan', u'Poznań'),
    ('kalisz', u'Kalisz'),
    ('konin', u'Konin'),
    ('leszno', u'Leszno'),
    ('chodzieski', u'chodziejski'),
    ('czarnkowsko-trzcianecki', u'czarnkowsko-trzcianecki'),
    ('gnieznienski', u'gnieźnieński'),
    ('gostynski', u'gostyński'),
    ('grodziski', u'grodziski'),
    ('jarocinski', u'jarociński'),
    ('kaliski', u'kaliski'),
    ('kepinski', u'kępiński'),
    ('kolski', u'kolski'),
    ('koninski', u'koniński'),
    ('koscianski', u'kościański'),
    ('krotoszynski', u'krotoszyński'),
    ('leszczynski', u'leszczyński'),
    ('miedzychodzki', u'międzychodzki'),
    ('nowotomyski', u'nowotomyski'),
    ('obornicki', u'obornicki'),
    ('ostrowski', u'ostrowski'),
    ('ostrzeszowski', u'ostrzeszowski'),
    ('pilski', u'pilski'),
    ('pleszewski', u'pleszewski'),
    ('poznanski', u'poznański'),
    ('rawicki', u'rawicki'),
    ('slupecki', u'słupecki'),
    ('szamotulski', u'szamotulski'),
    ('sredzki', u'średzki'),
    ('sremski', u'śremski'),
    ('turecki', u'turecki'),
    ('wagrowiecki', u'wągrowiecki'),
    ('wolsztynski', u'wolsztyński'),
    ('wrzesinski', u'wrzesiński'),
    ('zlotowski', u'złotowski'),
    ('bialogardzki', u'białogardzki'),
    ('choszczenski', u'choszczeński'),
    ('drawski', u'drawski'),
    ('goleniowski', u'goleniowski'),
    ('gryficki', u'gryficki'),
    ('gryfinski', u'gryfiński'),
    ('kamienski', u'kamieński'),
    ('kolobrzeski', u'kołobrzeski'),
    ('koszalinski', u'koszaliński'),
    ('lobeski', u'łobeski'),
    ('mysliborski', u'myśliborski'),
    ('policki', u'policki'),
    ('pyrzycki', u'pyrzycki'),
    ('slawienski', u'sławieński'),
    ('stargardzki', u'stargardzki'),
    ('szczecinecki', u'szczecinecki'),
    ('swidwinski', u'świdwiński'),
    ('walecki', u'wałecki'),
)

