{% extends "admin/base_site.html" %}
{% load i18n %}
{% load url from future %}

{% block breadcrumbs %}
<div class="breadcrumbs">
<a href="{% url 'admin:index' %}">{% trans 'Home' %}</a>
&rsaquo; <a href="{% url 'django-admindocs-docroot' %}">{% trans 'Documentation' %}</a>
&rsaquo; {% trans 'Templates' %}
&rsaquo; {{ name }}
</div>
{% endblock %}

{% block title %}Template: {{ name }}{% endblock %}

{% block content %}
<h1>Template: "{{ name }}"</h1>

{% regroup templates|dictsort:"site_id" by site as templates_by_site %}
{% for group in templates_by_site %}
    <h2>Search path for template "{{ name }}" on {{ group.grouper }}:</h2>
    <ol>
    {% for template in group.list|dictsort:"order" %}
        <li><code>{{ template.file }}</code>{% if not template.exists %} <em>(does not exist)</em>{% endif %}</li>
    {% endfor %}
    </ol>
{% endfor %}

<p class="small"><a href="{% url 'django-admindocs-docroot' %}">&lsaquo; Back to Documentation</a></p>
{% endblock %}
